@@ -43,7 +43,7 @@ functions, seeing and atmospheric conditions.
 """
 
 
-__version__ = "2.3.2"
+__version__ = "2.4"
 
 __all__ = [
     "MIN_MATCHES_FRACTION",
@@ -324,7 +324,6 @@ def find_transform(
 
     inv_model = _MatchTransform(source_controlp, target_controlp)
     n_invariants = len(matches)
-    max_iter = n_invariants
     # Set the minimum matches to be between 1 and 10 asterisms
     min_matches = max(1, min(10, int(n_invariants * MIN_MATCHES_FRACTION)))
     if (len(source_controlp) == 3 or len(target_controlp) == 3) and len(
@@ -334,7 +333,7 @@ def find_transform(
         inlier_ind = _np.arange(len(matches))  # All of the indices
     else:
         best_t, inlier_ind = _ransac(
-            matches, inv_model, 1, max_iter, PIXEL_TOL, min_matches
+            matches, inv_model, PIXEL_TOL, min_matches
         )
     triangle_inliers = matches[inlier_ind]
     d1, d2, d3 = triangle_inliers.shape
@@ -527,7 +526,7 @@ class MaxIterError(RuntimeError):
     pass
 
 
-def _ransac(data, model, min_data_points, max_iter, thresh, min_matches):
+def _ransac(data, model, thresh, min_matches):
     """fit model parameters to data using the RANSAC algorithm
 
     This implementation written from pseudocode found at
@@ -536,26 +535,23 @@ def _ransac(data, model, min_data_points, max_iter, thresh, min_matches):
     Given:
         data: a set of data points
         model: a model that can be fitted to data points
-        min_data_points: the minimum number of data values required to fit the
-            model
-        max_iter: the maximum number of iterations allowed in the algorithm
         thresh: a threshold value to determine when a data point fits a model
         min_matches: the min number of matches required to assert that a model
             fits well to data
     Return:
         bestfit: model parameters which best fit the data (or nil if no good
                   model is found)"""
-    iterations = 0
-    bestfit = None
-    best_inlier_idxs = None
+    good_fit = None
     n_data = data.shape[0]
-    n = min_data_points
     all_idxs = _np.arange(n_data)
+    _np.random.shuffle(all_idxs)
 
-    while iterations < max_iter:
+    for iter_i in range(n_data):
         # Partition indices into two random subsets
-        _np.random.shuffle(all_idxs)
-        maybe_idxs, test_idxs = all_idxs[:n], all_idxs[n:]
+        maybe_idxs = all_idxs[iter_i : iter_i + 1]
+        test_idxs = list(all_idxs[:iter_i])
+        test_idxs.extend(list(all_idxs[iter_i + 1 :]))
+        test_idxs = _np.array(test_idxs)
         maybeinliers = data[maybe_idxs, :]
         test_points = data[test_idxs, :]
         maybemodel = model.fit(maybeinliers)
@@ -564,15 +560,22 @@ def _ransac(data, model, min_data_points, max_iter, thresh, min_matches):
         also_idxs = test_idxs[test_err < thresh]
         alsoinliers = data[also_idxs, :]
         if len(alsoinliers) >= min_matches:
-            betterdata = _np.concatenate((maybeinliers, alsoinliers))
-            bestfit = model.fit(betterdata)
-            best_inlier_idxs = _np.concatenate((maybe_idxs, also_idxs))
+            good_data = _np.concatenate((maybeinliers, alsoinliers))
+            good_fit = model.fit(good_data)
             break
-        iterations += 1
-    if bestfit is None:
+
+    if good_fit is None:
         raise MaxIterError(
-            "Max iterations exceeded while trying to find "
-            "acceptable transformation."
+            "List of matching triangles exhausted before an acceptable "
+            "transformation was found"
         )
 
-    return bestfit, best_inlier_idxs
+    better_fit = good_fit
+    for i in range(3):
+        test_err = model.get_error(data, better_fit)
+        better_inlier_idxs = _np.arange(n_data)[test_err < thresh]
+        better_data = data[better_inlier_idxs]
+        better_fit = model.fit(better_data)
+    best_fit = better_fit
+    best_inlier_idxs = better_inlier_idxs
+    return best_fit, best_inlier_idxs
